///////////////////////////////////////////////////////////////////////////
//
//  XMLInterface.cc
// 
//  This interface allows maniuplation of xml strings in the following
//  style:
//  
//  XMLInterface foo = XMLInterface();
//  foo.Load(string_of_some_xml);
//  vector<int> nums = foo("iq")("channel").Get("PMT_Number")
//  vector<int> gains = foo("iq")("channel").Get("mVns")
//  
//
//  Version History:
//      
//      XX.XX.XXXX - Initial Submission (Michael Woods)
//

#include <iostream>
#include <string>

#include "XMLInterface.h"


#define VERBOSE_XI 0

XMLInterface::XMLInterface() {;}

XMLInterface::~XMLInterface() {;}

vector<string> XMLInterface::GetXML() {return the_xml;}

string XMLInterface::GetXMLString() {
    // Assemble the xml in the_xml to a single string and return it.
    concatenated_xml = "";
    for(size_t i=0; i<the_xml.size(); i++){
        concatenated_xml += the_xml[i];
    }
    return concatenated_xml;
}

void XMLInterface::Load(string some_xml) {
    // Stick the xml string into a vector and set it to the_xml.
    the_xml = vector<string>(1,some_xml);
}

XMLInterface XMLInterface::operator() (string search_term) {

    vector<string> result_vector;


    string start_tag = "<" + search_term + ">";
    string end_tag  = "</" + search_term + ">"; // </PMT_number>

    bool found_end = false;
    found_end = found_end;

    for(size_t x=0; x<the_xml.size(); x++) {
        string xml = the_xml[x].c_str();

        size_t start_tag_pos = 0;       // <PMT_number> Where is the "<"?
        size_t end_tag_pos   = 0;       // </PMT_number> Where is the "<"?
        size_t last_pos      = 0;

        size_t size = xml.size();

        size_t number_start;
        size_t number_end;
        string a_number;


        while(true) {
            start_tag_pos = xml.find(start_tag, last_pos+1);
            end_tag_pos = xml.find(end_tag, start_tag_pos);

            // Make sure to exit if we are done.
            if(start_tag_pos > size) break;

            number_start = start_tag_pos + start_tag.size();
            number_end   = end_tag_pos;
            last_pos = start_tag_pos;

            a_number = xml.substr(number_start, number_end-number_start);
            
            result_vector.push_back(a_number);
            
        }
    }

    string end_result;
    for(size_t i=0; i<result_vector.size(); i++) {
        end_result += result_vector[i];
    }
    
    // For the time being, none of the below ideas are good. Let's do this
    // instead (I know I"m goign to hate myself for this brevity later).
    XMLInterface temp = XMLInterface();
    temp.Load(end_result);
    return temp;


    // This is a test until I get it working.
    //XMLInterface a_new_interface = XMLInterface();
    //a_new_interface.Load(end_result);
    //return a_new_interface;

    // This too!
    //return *this


    // This is what I want to happend:
    //the_xml = end_result;
    //return this;
}
